<?php
// $Id: EntityFetcherAbstract.php,v 1.1 2010/03/24 00:45:43 pounard Exp $

class Yamm_EntityFetcherException extends Yamm_EntityException {}


/**
 * Get dependencies abstraction
 *
 * TODO implement initial pull
 */
abstract class Yamm_EntityFetcherAbstract {

  /**
   * Do initial pull
   *
   * @return mixed
   *   Array of entities
   *   FALSE when no entities from server
   */
  public function pull() {
    $entities = $this->_pull();

    if (! is_array($entities)) {
      throw new Yamm_EntityFetcherException("No result from fetcher");
    }

    return $entities;
  }

  /**
   * Fetch all entity dependencies
   *
   * @param Yamm_Entity $entity
   * @return array
   *   Array of Yamm_Entity instances
   */
  public function fetchDependencies(array $dependencies) {
    // Shortcut to avoid fetcher implementation try to fetch empty data
    if (empty($dependencies)) {
      return array();
    }

    $entities = $this->_fetchDependencies($dependencies);

    if (! is_array($entities)) {
      throw new Yamm_EntityFetcherException("No result from fetcher");
    }

    return $entities;
  }

  /**
   * Override this method with your business stuff
   *
   * @return array
   *   Array of entities
   */
  protected abstract function _pull();

  /**
   * Override this method with your business stuff
   *
   * @param array &$uuid_array
   *   Array of UUID string
   * @return array
   *   Array of entities
   */
  protected abstract function _fetchDependencies(array &$uuid_array);
}
