<?php
// $Id: Transaction.php,v 1.2 2010/03/25 23:53:11 pounard Exp $

/**
 * @file
 * Yamm client synchronization job implementation. This no longer relies on
 * DataSync module.
 */

/**
 * Custom DataSync job implementation
 */
class Yamm_Transaction {

  public function run($data) {
    // Check we have a transaction in "running" state
    if ($this->_yammTid = $data->tid) {
      yamm_client_transaction_update_status($this->_yammTid, YAMM_TRANSACTION_STATUS_RUNNING);
      $this->_yammServerUrl = $data->server_url;
      return $this->syncAllContent();
    }
    else {
      $this->sendStatus(YAMM_TRANSACTION_STATUS_CANCELED);
      return FALSE;
    }
  }

  public function cancel() {
    $this->sendStatus(YAMM_TRANSACTION_STATUS_CANCELED);
  }

  public function fail($method = NULL) {
    $this->sendStatus(YAMM_TRANSACTION_STATUS_CANCELED);
  }

  public function sendStatus($status) {
    // Close transaction on client
    yamm_client_transaction_update_status($this->_yammTid, $status);

    // Notify server in case we have its url
    if ($this->_yammServerUrl) {
      $this->sendCall('yamm.client.status', $status);
    }
  }

  /**
   * Do an xmlrpc call to server
   *
   * @see yamm_api_xmlrpc_call()
   *
   * @param string $method
   *   Method to cal
   * @param ..
   *   Arbitrary parameters you want to send to
   * @return mixed
   *   Same result as yamm_api_xmlrpc_call()
   */
  public function sendCall($method) {
    if ($this->_yammServerUrl) {
      $args = func_get_args();
      $method = array_shift($args);
      array_unshift($args, $this->_yammTid);
      array_unshift($args, $method);
      array_unshift($args, $this->_yammServerUrl);
      return call_user_func_array('yamm_api_xmlrpc_call', $args);
    }
    return FALSE;
  }

  /**
   * Server URL
   * @var string
   */
  protected $_yammServerUrl = NULL;

  /**
   * Transaction identifier with server
   * @var string
   */
  protected $_yammTid = NULL;

  /**
   * One and only phase, content synchronisation
   */
  public function syncAllContent() {
    yamm_api_debug("Transaction " . $this->_yammTid . " running");

    try {
      $parser = new Yamm_EntityParser(
        new Yamm_EntityFetcher_Xmlrpc(
          $this->_yammServerUrl, $this->_yammTid));

      $parser->parse();

      module_invoke_all('yamm_sync_finished');

      drupal_flush_all_caches();

      $this->sendStatus(YAMM_TRANSACTION_STATUS_FINISHED);
      return TRUE;
    }
    catch (Yamm_EntityFetcherException $e) {
      yamm_api_debug($e->getMessage());
      yamm_api_debug($e->getTraceAsString());
      // If we dont change status here, no other transanctions this client wont
      // be able to run any more transaction.
      $this->cancel();
    }
    catch (Yamm_EntityException $e) {
      yamm_api_debug($e->getMessage());
      yamm_api_debug($e->getTraceAsString());
      // If we dont change status here, no other transanctions this client wont
      // be able to run any more transaction.
      $this->fail();
    }

    return FALSE;
  }
}
